#include <Arduino.h>
#include <AudioOutputI2S.h>
#include <AudioGenerator.h>
#include <AudioGeneratorMilena.h>
#include <freertos/ringbuf.h>
#include <stdarg.h>

#include "Poziomica.h"


static int siglen[51]={
200, 195, 189, 184, 179, 174, 169, 164, 159, 155, 150, 146, 142, 138,
134, 130, 126, 123, 119, 116, 113, 109, 106, 103, 100, 97, 95, 92, 89,
87, 84, 82, 80, 77, 75, 73, 71, 69, 67, 65, 63, 61, 60, 58, 56, 55, 53,
52, 50};

static float sigfreq[51]={
0.50, 0.51, 0.53, 0.55, 0.56, 0.58, 0.59, 0.61, 0.63, 0.65, 0.67, 0.69,
0.71, 0.73, 0.75, 0.77, 0.79, 0.82, 0.84, 0.87, 0.89, 0.92, 0.94, 0.97,
1.00, 1.03, 1.06, 1.09, 1.12, 1.16, 1.19, 1.22, 1.26, 1.30, 1.33, 1.37,
1.41, 1.46, 1.50, 1.54, 1.59, 1.63, 1.68, 1.73, 1.78, 1.83, 1.89, 1.94,
2.00};

static float sigain[51]={
    0.10, 0.10, 0.11, 0.12, 0.12, 0.13, 0.13, 0.14, 0.15, 0.15, 0.16,
    0.17, 0.18, 0.19, 0.20, 0.21, 0.22, 0.23, 0.24, 0.25, 0.26, 0.27,
    0.29, 0.30, 0.32, 0.33, 0.35, 0.37, 0.38, 0.40, 0.42, 0.44, 0.46,
    0.49, 0.51, 0.54, 0.56, 0.59, 0.62, 0.65, 0.68, 0.71, 0.75, 0.79,
    0.83, 0.87, 0.91, 0.95, 1.00
};

class AudioGeneratorBeep : public AudioGenerator
{
    public:

    bool begin(int freq, int duration, AudioOutputI2S *out);
    bool stop();
    bool isRunning();
    bool loop();
    
    private:
    int phase;
    int beeplen;
    int beepvol;
    int samplen;
};

bool AudioGeneratorBeep::begin(int freq, int duration, AudioOutputI2S *out)
{
    phase = 0;
    beepvol =
#ifdef EXTERNAL_DAC
    1000
#else
    5000
#endif    
     ;
    samplen = freq;
    if (duration < 25) duration = 25;
    beeplen = duration * 44;
    output=out;
    running=true;
    out->SetRate(44100);
    out->SetBitsPerSample(16);
    out->SetChannels(1);
    return out->begin();
}

bool AudioGeneratorBeep::stop()
{
    if (!running)
        return true;
    running = false;
    output->stop();
    return true;
}

bool AudioGeneratorBeep::isRunning()
{
    return running;
}

bool AudioGeneratorBeep::loop(void)
{
    int i;
    for (i=0;i<4410 && running;i++) {
        if (phase >= beeplen) {
            stop();
            break;
        }
        int pos = phase % samplen;
        int16_t sample =
#ifdef EXTERNAL_DAC
        (pos > samplen / 2) ? beepvol : -beepvol;
#else    
         (pos > samplen / 2) ? -32768 + 2 * beepvol : -32768;
#endif
        lastSample[AudioOutput::LEFTCHANNEL] =
            lastSample[AudioOutput::RIGHTCHANNEL] = sample;
        if (!output->ConsumeSample(lastSample))
            break;
        phase++;
    }
    return running;
}


static AudioGeneratorMilena *mil;
static AudioOutputI2S *player;
static AudioGeneratorBeep *beeper;

int myPitch, mySpeed, myContrast,myGain=0;
static char myDeciPoint[16];
static bool myDeciMode;
static bool myColonMode;

static volatile bool is_Speaking;
static void initAudioOutput(void)
{
#ifdef EXTERNAL_DAC
    player = new AudioOutputI2S();
    player->SetPinout(PIN_BCLK, PIN_WCLK, PIN_DOUT);
    player->SetGain(sigain[myGain]);
#else
    player = new AudioOutputI2S(0,1);
#ifdef USE_GAIN
    player->SetGain(sigain[myGain]);
#else
    player->SetGain(1.0);
#endif
#endif
}

static void stopAll(bool keep_speaking=false)
{
    if (beeper) {
        beeper->stop();
        delete beeper;
        beeper = NULL;
    }
    if (mil) {
        mil->stop();
        delete mil;
        mil = NULL;
    }
    if (player) {
        delete(player);
        player = NULL;
    }
    if (!keep_speaking) is_Speaking=false;
}

bool startBeep(int nfreq, int duration)
{
    stopAll(false);
    initAudioOutput();
    beeper = new AudioGeneratorBeep();
    nfreq=constrain(nfreq,-24,24);
    if (!beeper->begin(siglen[nfreq+24], duration, player)) {
        
        delete beeper;
        delete player;
        beeper = NULL;
        player = NULL;
        return false;
    }
    return is_Speaking = true;
}

bool startTalking(const char *txt)
{
    stopAll(true);
    mil=new AudioGeneratorMilena();
    if (!mil) return false;
    initAudioOutput();
    if (!player) {
        delete mil;
        return false;
    }
    mil->setSpeed(sigfreq[mySpeed+24]);
    mil->setPitch(sigfreq[myPitch+24]);
    mil->setContrast(myContrast);
    mil->setSimpleDeciPoint(myDeciMode);
    mil->setDeciPoint(myDeciPoint);
    mil->setAltColon(1);
    
#ifndef EXTERNAL_DAC
    mil->setVolume(1.4);
    mil->setInternalDAC(1);
#endif
    //mil->setShowPhonemes(1);
    if (!mil->begin(txt, player)) {
        delete mil;
        delete player;
        mil = NULL;
        player = NULL;
        return is_Speaking = false;
    }
    return is_Speaking = true;
    
}

void speakerloop()
{
    
    if (beeper && beeper->isRunning()) {
        if (!beeper->loop()) {
            stopAll();
        }
        else delay(1);
    }
    if (mil && mil->isRunning()) {
        if (!mil->loop()) {
            stopAll();
        }
        else delay(1);
    }
}

static RingbufHandle_t SpeechRB;
TaskHandle_t speechHandle;

struct SpeakerCmd {
    uint8_t command;
    union {
        char txt[127];
        struct {
            int8_t freq;
            int8_t duration;
        } s;
    } u;
};

enum {
    SCMD_STOP = 0,
    SCMD_SAY,
    SCMD_BEEP,
    SCMD_SAYCST
};

void speechThread(void *dummy)
{
    struct SpeakerCmd *rc;
    size_t itemsize;
    char *txtadr;
    for (;;) {
        speakerloop();
        rc = (struct SpeakerCmd *)xRingbufferReceive(SpeechRB, &itemsize, is_Speaking ? 0 : portMAX_DELAY);
        if (!rc) continue;
        switch (rc->command) {
            case SCMD_STOP:
            stopAll();
            break;

            case SCMD_BEEP:
            startBeep(rc->u.s.freq, rc->u.s.duration);
            break;

            case SCMD_SAY:
            startTalking(rc->u.txt);
            break;

            case SCMD_SAYCST:
            memcpy(&txtadr,rc->u.txt,sizeof(txtadr));
            startTalking(txtadr);
            break;

        }
        vRingbufferReturnItem(SpeechRB, (void *)rc);
    }
}

void stopAudio()
{
    uint8_t scmd;
    scmd = SCMD_STOP;
    xRingbufferSend(SpeechRB, &scmd, 1, 0);

}


void startTalkerTask(void)
{
    SpeechRB = xRingbufferCreateNoSplit(256,4);
    xTaskCreatePinnedToCore(speechThread,"speech",10000,NULL,3,&speechHandle,0);
}

bool isSpeaking(void)
{
    return is_Speaking;
    
}

void say(const char *txt)
{
    struct SpeakerCmd scmd;
    if (strlen(txt) > 126) return ;
    scmd.command = SCMD_SAY;
    strcpy(scmd.u.txt,txt);
    is_Speaking=true;
    printf("GADACZ %s\n", txt);
    xRingbufferSend(SpeechRB, &scmd, strlen(txt)+2, 0);
    
}

void saycst(const char *txt)
{
    uint8_t scmd[1+sizeof(txt)];
    scmd[0] = SCMD_SAYCST;
    memcpy(&scmd[1], &txt, sizeof(txt));
    is_Speaking=true;
    printf("GADACZ %s\n", txt);
    xRingbufferSend(SpeechRB, &scmd, sizeof(txt)+1, 0);
}

void sayfmt(const char *format, ...)
{
    va_list ap;
    struct SpeakerCmd scmd;
    int size=126;
    va_start(ap, format);
    size = vsnprintf(scmd.u.txt, size, format, ap);
    va_end(ap);
    if (size > 126) return;
    scmd.command = SCMD_SAY;
    is_Speaking=true;
    printf("GADACZ %s\n", scmd.u.txt);
    
    xRingbufferSend(SpeechRB, &scmd, size+2, 0);
}


void beep(int freq)
{
    struct {
        uint8_t cmd;
        int8_t freq;
        int8_t duration;
    } bip;
    bip.cmd = SCMD_BEEP;
    bip.freq = freq;
    bip.duration = 35;
    xRingbufferSend(SpeechRB, &bip, 3, 0);
}

        
void setSpeed(int n, bool nobeep)
{
    if (n < -24 || n > 24) return;
    mySpeed = n;
    if (!nobeep) beep(n);
}

void setPitch(int n, bool nobeep)
{
    if (n < -24 || n > 24) return;
    myPitch = n;
    if (!nobeep) beep(n);
}
    
void setGain(int n, bool nobeep)
{
    if (n < 0 || n > 48) return;
    myGain = n;
    if (!nobeep) beep(n-24);
}

void setContrast(int n, bool nobeep)
{
    if (n < 0 || n > 48) return;
    myContrast = 2* n;
    if (!nobeep) beep(n-24);
}

int getPitch(void) {return myPitch;}
int getSpeed(void) {return mySpeed;}
int getContrast(void) {return myContrast;}
int getGain(void) {return myGain;}

void changeSpeed(int n)
{
    int ns = mySpeed + n;
    ns=constrain(ns,-24,24);
    if (ns == mySpeed) return;
    mySpeed = ns;
    beep(mySpeed);
}

void changePitch(int n)
{
    int ns = myPitch + n;
    ns=constrain(ns,-24,24);
    if (ns == myPitch) return;
    myPitch = ns;
    beep(myPitch);
}

void changeGain(int n)
{
    int ns = myGain + n;
    ns=constrain(ns,0,48);
    if (ns == myGain) return;
    myGain = ns;
    beep(myGain-24);
}

void changeContrast(int n)
{
    int ns = myContrast + n;
    ns=constrain(ns,0,48);
    if (ns == myContrast) return;
    myContrast = ns;
    beep(myContrast-24);
}

void initSpeechParams(void)
{
    myPitch=preferences.getInt("pitch",0);
    mySpeed=preferences.getInt("speed",0);
    myContrast=preferences.getInt("contrast",0);
    myGain=preferences.getInt("gain",0);
    myDeciMode = preferences.getBool("decimode", false);
    myColonMode = preferences.getBool("colonmode", false);
    if (!preferences.getString("decipoint",myDeciPoint,15)) strcpy(myDeciPoint,"precinek");
}

bool getDPMode(void)
{
    return myDeciMode;
}

uint8_t getDPStrN(const char **strs)
{
    int i;
    for (i=0; strs[i]; i++) if (!strcmp(myDeciPoint,strs[i])) return i;
    return 0;
}

void setDPMode(bool mode)
{
    myDeciMode = mode;
    if (preferences.getBool("decimode", false) != mode)
        preferences.putBool("decimode",mode);
}

bool getColonMode(void)
{
    return myColonMode;
}

void setColonMode(bool mode)
{
    myColonMode = mode;
    if (preferences.getBool("colonmode", false) != mode)
        preferences.putBool("colonmode",mode);
}

void setDPStr(const char *str)
{
    if (!preferences.getString("decipoint", myDeciPoint,15) || strcmp(myDeciPoint, str)) {
        strcpy(myDeciPoint,str);
        preferences.putString("decipoint",str);
    }
}


void storeSpeechParams(void)
{
    if (preferences.getInt("pitch",0) != myPitch) preferences.putInt("pitch", myPitch);
    if (preferences.getInt("speed",0) != mySpeed) preferences.putInt("speed", mySpeed);
    if (preferences.getInt("contrast",0) != myContrast) preferences.putInt("contrast", myContrast);
    if (preferences.getInt("gain",0) != myGain) preferences.putInt("gain", myGain);
}

