#include <Wire.h>
#include "Poziomica.h"


Bounce keybounce[3];
int crosstalk = CROSSX_DISTANCE;
uint32_t lastUsage;
uint8_t mainMode, subMode;
static uint8_t lastMainMode, lastSubMode;
static uint8_t anoBatt =0;

int coverdim[5] = {
    COVER_LEN,
    COVER_WIDTH,
    COVER_HEIGHT,
    COVER_OFFSET,
    VERT_OFFSET};

#define cover_len coverdim[0]
#define cover_width coverdim[1]
#define cover_height coverdim[2]
#define cover_offset coverdim[3]
#define vert_offset coverdim[4]
#define top_offset (coverdim[2] - coverdim[4])

int std_dims[5]={COVER_LEN, COVER_WIDTH, COVER_HEIGHT, COVER_OFFSET, VERT_OFFSET};
    
Preferences preferences;

extern void scanner();
extern void initMPU(void);

#ifdef PIN_VOLT
#define MAX_ANOBAT 4
#else
#define MAX_ANOBAT 3
#endif

enum {
    SEPR_DECIPOINT,
    SEPR_SIMPDECI,
    SEPR_COLONMODE,
    SEPR_BATTERY,
    SEPR_MAX};

enum {
    SPKL_SPKMODE = 0,
    SPKL_SPKAXIS,
    SPKL_MAX};
enum {
    SMEN_SPEECH = 0,
    SMEN_PRON,
    SMEN_LEVEL,
#ifdef USE_GLASS
    SMEN_LASER,
#endif
    SMEN_GYRO,
    SMEN_MAX
};

enum {
    SELV_START=0,
    SELV_WAIT1,
    SELV_COUNTDOWN1,
    SELV_CALIBXY,
    SELV_WAIT2,
    SELV_COUNTDOWN2,
    SELV_CALIBXY2,
    SELV_EOC,
    SELV_EOC1,
    SELV_EOC2,
    SELV_WAIT3,
    SELV_COUNTDOWN3,
    SELV_CALIBZ,
    SELV_WAIT4,
    SELV_COUNTDOWN4,
    SELV_CALIBZ2,
    SELV_EOZ,
    SELV_FINALE
    
    };

enum {
    SEGY_START = 0,
    SEGY_WAIT,
    SEGY_COUNTDOWN,
    SEGY_CALIBRATE,
    SEGY_FINALE
};

enum {
    SELA_START = 0,
    SELA_WAIT1,
    SELA_SPKCAL1,
    SELA_WAITCAL1,
    SELA_WAITCAL1A,
    SELA_WAITCAL1B,
    SELA_COUNTDOWN1,
    SELA_SPADOK,
    SELA_SPADOK1,
    SELA_SPADOK2,
    SELA_CALIBOF,
    SELA_OFFOK,
    SELA_OFFOK1,
    SELA_OFFOK2,
    SELA_CROCNT,
    SELA_DONE,
    SELA_FINALE,
    SELA_CONFIRM,
    SELA_MAX
};

enum {
    MMODE_LEVEL=0,
    MMODE_LIDAR,
    MMODE_MENU,
    MMODE_SPEECHS,
    MMODE_ZEROS,
    MMODE_MAX
};

enum {
    MSPE_PITCH=0,
    MSPE_SPEED,
    MSPE_CONTRAST,
    MSPE_GAIN,
    MSPE_MAX};

enum {
    SPKA_SPKMODE=0,
    SPKA_SPKSIMPLE,
#ifdef HAVE_NARROW
    SPKA_SPKANGLE,
#endif
    SPKA_POINT,
    SPKA_DISTAN,
    SPKA_CORRECT,
    SPKA_MAX};
    


static void restoreCurState(void);

void setup(void)
{
    int i;
    Serial.begin(115200);
    preferences.begin("poziomica");
    anoBatt = preferences.getInt("anobat",0);
    if (anoBatt >= MAX_ANOBAT) anoBatt = 0;
    if (preferences.getBytes("coverdim",coverdim, sizeof(coverdim)) != sizeof(coverdim)) {
        for (i=0; i<5; i++) coverdim[i] = std_dims[i];
    }
    crosstalk=preferences.getInt("crosstalk", CROSSX_DISTANCE);
    initBattery();
    initSpeechParams();
    startTalkerTask();
    keybounce[0].attach(PIN_RIGHT, INPUT_PULLUP);
    keybounce[1].attach(PIN_MID, INPUT_PULLUP);
    keybounce[2].attach(PIN_LEFT, INPUT_PULLUP);
    keybounce[0].interval(50);
    keybounce[1].interval(50);
    keybounce[2].interval(50);
    keybounce[0].update();
    keybounce[1].update();
    keybounce[2].update();
    mainMode = 0;
    subMode = 0;
    restoreCurState();
    lastSubMode = 0xff;
    Wire.begin(PIN_SDA, PIN_SCL);
    Wire.setClock(400000);
    delay(100);
    //initEnPairCentral("Poziomica","5G-Ultra-Power-Test","Aniepowiembonie");
    initMPU();
    //scanner();
    initLidar();
    lastUsage=millis();
}

const char *spModeNames[]={
    "ustawienie wysokości głosu: %d",
    "ustawienie tempa głosu: %d",
    "ustawienie kontrastu głosu: %d",
    "ustawienie głośności: %d"};

const char *spModeRes[] = {
    "Wysokość głosu %d",
    "Tempo głosu %d",
    "Efekt kontrastu %d",
    "Głośność mowy %d"};

typedef int (*getterFun)(void);
typedef void (*changerFun)(int);

getterFun spModeGetFun[] ={
    getPitch,
    getSpeed,
    getContrast,
    getGain};

changerFun spModeChgFun[] ={
    changePitch,
    changeSpeed,
    changeContrast,
    changeGain};

static bool settingMode;
static int settingMenu, settingSub, settingMenuLast, settingSubLast;


void announceMode()
{
    sayfmt(spModeNames[settingSub],spModeGetFun[settingSub]());
}


uint32_t lastTick[3];
uint8_t tickMode[3];
uint8_t keyStatus[3];

void update_repeatable(int nr)
{
    keyStatus[nr] = 0;
    if (keybounce[nr].fell()) {
        keyStatus[nr] = 1;
        lastTick[nr] = millis();
        tickMode[nr] = 1;
        return;
    }
    if (keybounce[nr].rose()) {
        tickMode[nr] = 15; //
        lastTick[nr] = millis();
        return;
    }
    if (keybounce[nr].read()) {
        if (tickMode[nr] == 15 && millis() - lastTick[nr] > 1000) {
            keyStatus[nr] = 3;
            tickMode[nr] = 0;
        }
        return;
    }
    // klawisz wciśnięty
    uint32_t delta = millis() - lastTick[nr];
    if (tickMode[nr] == 1) {
        if (delta > 500UL) {
            tickMode[nr] = 2;
            keyStatus[nr] = 1;
            lastTick[nr] = millis();
        }
        return;
    }
    if (delta < 150UL) return;
    if (tickMode[nr] < 6) {
        tickMode[nr]++;
        keyStatus[nr] = 1;
    }
    else {
        keyStatus[nr] = 2;
    }
    lastTick[nr] = millis();
}


void do_SpeechMenu()
{
    if (settingMenuLast != settingMenu || settingSubLast != settingSub) {
        settingSubLast = settingSub;
        settingMenuLast = settingMenu;
        announceMode();
    }
    if (keybounce[1].rose()) {
        storeSpeechParams();
        settingSub++;
#ifndef EXTERNAL_DAC
#ifndef USE_GAIN
        if (settingSub == MSPE_GAIN) settingSub ++;
#endif
#endif        
        settingSub %= MSPE_MAX;
        return;
    }
    update_repeatable(0);
    update_repeatable(2);
    if (keyStatus[0] == 3 || keyStatus[2] == 3) {
        sayfmt(spModeRes[settingSub], spModeGetFun[settingSub]());
        return;
    }
    int n=0;
    if (keyStatus[0] == 1) n = -1;
    else if (keyStatus[0] == 2) n = -4;
    else if (keyStatus[2] == 1) n = 1;
    else if (keyStatus[2] == 2) n = 4;
    if (n) {
        spModeChgFun[settingSub](n);
    }
}


const char *pronDectStrs[] = {
    "przecinek","kropka","i",NULL};
const char *batStatStr[] = {
    "co dwie minuty",
    "co dziesięć minut",
#ifdef PIN_VOLT
    "tylko rozładowana",
#endif
    "nigdy"};


bool deciPointMode;
bool colonMode;
uint8_t deciPointStr;
uint8_t newAnoBatt;
void announcePron(void)
{
    switch(settingSub) {
        case SEPR_DECIPOINT:
        sayfmt("Separator dziesiętny: %s", pronDectStrs[deciPointStr]);
        break;

        case SEPR_SIMPDECI:
        sayfmt("Cyfra po przecinku: %s",deciPointMode ? "uproszczona": "pełna");
        break;

        case SEPR_COLONMODE:
        sayfmt("Intonacja dwukropka %s", colonMode ? ", odwrotna" : ": normalna");
        break;
        
        case SEPR_BATTERY:
#ifdef PIN_VOLT
        sayfmt("Stan baterii: %s", batStatStr[newAnoBatt]);
#else
        sayfmt("Przypomnienie o wyłączeniu: %s",batStatStr[newAnoBatt]);
#endif
        break;
    }
}


void doSettingsPron()
{
    if (settingMenuLast != settingMenu) {
        deciPointMode = getDPMode();
        colonMode = getColonMode();
        deciPointStr = getDPStrN(pronDectStrs);
        newAnoBatt = anoBatt;
        
    }
    if (settingMenuLast != settingMenu || settingSubLast != settingSub) {
        settingSubLast = settingSub;
        settingMenuLast = settingMenu;
        announcePron();
    }
    if (keybounce[1].rose()) {
        switch (settingSub) {
            case SEPR_DECIPOINT:
            setDPStr(pronDectStrs[deciPointStr]);
            break;

            case SEPR_SIMPDECI:
            setDPMode(deciPointMode);
            break;

            case SEPR_COLONMODE:
            setColonMode(colonMode);
            break;

            case SEPR_BATTERY:
            if (anoBatt != newAnoBatt) {
                anoBatt = newAnoBatt;
                preferences.putInt("anobat", anoBatt);
            }
            break;
        }
        settingSub++;
        settingSub %= SEPR_MAX;
        return;
    }

    if (keybounce[2].rose() || keybounce[0].rose()) {
        switch (settingSub) {
            case SEPR_SIMPDECI:
            deciPointMode = !deciPointMode;
            announcePron();
            break;

            case SEPR_COLONMODE:
            colonMode = !colonMode;
            announcePron();
            break;

            case SEPR_BATTERY:
            newAnoBatt = (newAnoBatt + 1) % MAX_ANOBAT;
            announcePron();
            break;
            
            case SEPR_DECIPOINT:
            deciPointStr = (deciPointStr +1) % 3;
            announcePron();
            break;
        }
    }
            
    
}

void simpledistan(int distan)
{
    if (distan < 50) {
        sayfmt("%d mm",distan);
        return;
    }
    if (distan <100) {
        distan = (distan+2) / 5;
        if (!(distan & 1)) {
            sayfmt("%d cm", distan / 2);
        }
        else {
            sayfmt("%d u pół centymetra", distan/2);
        }
        return;
    }
    distan = (distan+5) / 10;
    if (distan < 100) {
        sayfmt("%d", distan);
        return;
    }
    if (distan < 200) {
        if (distan == 100) say("metr");
        else sayfmt("metr %d", distan-100);
        return;
    }
    distan += 5;
    int metry = distan / 100;
    distan = (distan % 100);
    distan -= distan % 10;
    if (!distan) {
        sayfmt("%d m", metry);
    }
    else {
        sayfmt("%d %d", metry, distan);
    }
}


static uint32_t sms;


uint8_t lidrmode[6]; // ondemand, simple, narrow, fromback, distance, correct
const char *lidrval[6][3]={
    {"ciągły", "na żądanie"},
    {"dokładny", "pobieżny"},
#ifdef HAVE_NARROW
    {"wąska","szeroka"},
#endif
    {"od przodu", "od tyłu"},
    {"długi","średni","krótki"},
    {"brak","poziomica","żyroskop"}};
const char *lidrfmt[]={
    "odczyt dalmierza: %s",
    "sposób czytania: %s",
#ifdef HAVE_NARROW
    "wiązka: %s",
#endif
    "pomiar: %s",
    "dystans: %s",
    "korekta: %s",};
void announceLidar(void)
{
    sayfmt(lidrfmt[subMode], lidrval[subMode][lidrmode[subMode]]);
    sms=millis();
}

uint8_t orientation,lorient=0xff;
float anglex, angley;

void speakLidar(bool force=0)
{
    if (!force) {
        if (isSpeaking()) {
            sms = 0;
            return;
        }
        if (!sms) {
            sms=millis();
            return;
        }
        if (millis() - sms < 500UL) return;
        if (!keybounce[0].read() || !keybounce[1].read() || !keybounce[2].read()) return;
    }
    int d=getDistance();
    if (d < 0) {
        if (force) {
            lastUsage = millis();
            saycst("Błąd odczytu");
        }
        return;
    }
    lastUsage = millis();
    if (d < 0) {
        saycst("Błąd odczytu");
    }
    else {
        float dreal;
        
        if (lidrmode[SPKA_CORRECT]) {
            float gmpx,fadd=0,cosa;
            //float d1, d2, ha,sina,cosa;
            //static float t13 = tan(2 * 3.141592 / 180);

            if (lidrmode[SPKA_CORRECT] == 1) {
                orientation=getLevel(&anglex, &angley, &gmpx);
                if (lidrmode[SPKA_POINT]) {
                    switch(orientation) {
                        case 0:
                        case 4:
                        if (gmpx > 0) fadd = (top_offset) * sin(gmpx * 3.141592 / 180);
                        else fadd = (vert_offset) * sin(-gmpx * 3.141592 / 180);
                        break;

                        case 1:
                        case 5:
                        fadd = cover_width * 2.0 * sin(fabs(gmpx) * 3.141592 / 180);
                        break;
                        


                        case 2:
                        
                        if (anglex > 0) fadd = vert_offset * sin(anglex * 3.141592 / 180);
                        else fadd = (top_offset) * sin(-anglex * 3.141592 / 180);
                        fadd += cover_width / 2.0 * sin(fabs(angley) * 3.141592 / 180);
                        break;

                        case 6:
                        if (anglex < 0) fadd = vert_offset * sin(-anglex * 3.141592 / 180);
                        else fadd = (top_offset) * sin(anglex * 3.141592 / 180);
                        fadd += cover_width / 2.0 * sin(fabs(angley) * 3.141592 / 180);
                        break;
                    }
                }
                        
            } else {
                orientation=getLevel(&anglex, &angley, NULL);
                if ((orientation & 3) == 0) {
                    gmpx = fmod(fabs(getGyroZ()), 90.0);
                    if (gmpx >= 45.0) gmpx = 90.0 - gmpx;
                    if (lidrmode[SPKA_POINT]) fadd = cover_width / 2.0 * sin(gmpx * 3.141592 / 180);
                }
                else if ((orientation & 3) == 1) {
                    gmpx = fmod(getGyroZ(1),90.0);
                    if (gmpx >= 45.0) gmpx = gmpx - 90.0;
                    else if (gmpx <= -45.0) gmpx = 90.0+gmpx;
                    if (lidrmode[SPKA_POINT]) fadd = ((gmpx < 0) ? (top_offset):vert_offset) / 2.0 * sin(gmpx * 3.141592 / 180);
                }
            }
            //sina = sin(fabs(gmpx * 3.141592 / 180.0));
            cosa = cos(gmpx * 3.141592 / 180.0);
            //ha = d * t13;
            //d2 = ha * sina;
            //dreal = d + d2;
            dreal = d;
#ifndef USE_GLASS
            dreal -= cover_offset;
#endif
            if (lidrmode[SPKA_POINT]) dreal += cover_len;
            int de=dreal * cosa + fadd;
            sayfmt("%s: %d mm, %s %.1f°",lidrmode[SPKA_POINT]?"od tyłu":"od przodu", de,
                (lidrmode[SPKA_CORRECT] == 2) ? "żyro" :
                ((orientation & 3)  == 2) ? "pion" : "poziom",
                gmpx
                );
            
            
        }
        else {
#ifndef USE_GLASS
            if (d >= cover_offset) d -= cover_offset;
#endif
            if (lidrmode[SPKA_POINT]) d += cover_len;
            if (lidrmode[SPKA_SPKSIMPLE]) simpledistan(d);
            else sayfmt("%s: %d mm",lidrmode[SPKA_POINT]?"od tyłu":"od przodu", d);
        }
    }
    sms=millis();
}


int unitform(int n)
{
    if (n == 1) return 0;
    n %= 100;
    if (n>=2 && n<=4) return 1;
    if (n<20) return 2;
    n %= 10;
    if (n >= 2 || n <= 4) return 1;
    return 2;
}

uint8_t levlmode[2];
    

void speakLevel(bool force=false)
{
    static const char * orients[] = {
        "pozioma",
        "boczna",
        "pionowa"};
    if (!force) {
        if (isSpeaking()) {
            sms = 0;
            return;
        }
        if (!sms) {
            sms=millis();
            return;
        }
        if (millis() - sms < 500UL) return;
        if (!keybounce[0].read() || !keybounce[1].read() || !keybounce[2].read()) return;
    }
    orientation=getLevel(&anglex, &angley);
    lastUsage = millis();
    if (lorient != orientation) {
        lorient = orientation;
        sayfmt("Orientacja %s%s",orients[orientation & 3], (orientation & 4) ? " odwrócona":"");
    }
    else {
#if 0        
        char buf[128],*c;
        strcpy(buf,"Kąt "); c = buf + strlen(buf);
        c = dtocolq(c,anglex,NULL);
        if (!levlmode[SPKL_SPKAXIS]) {
            strcat(c," na ");
            c += strlen(c);
            c = dtocolq(c,angley,NULL);
        }
        say(buf);
#else
        if (levlmode[SPKL_SPKAXIS]) sayfmt("Kąt %.1f",anglex);
        else sayfmt("Kąt %.1f na %.1f", anglex, angley);
#endif
    }
    sms = millis();
}

const char *levlval[][2]={
    {"ciągły", "na żądanie"},
    {"obie osie","jedna oś"}};
    
const char *levlfmt[]={
    "odczyt kąta: %s",
    "sposób czytania: %s"};

void announceLevel(void)
{
    sayfmt(levlfmt[subMode], levlval[subMode][levlmode[subMode] & 1]);
    sms=millis();
}

void doSpeakLevel(bool force=false)
{
    if (!levlmode[SPKL_SPKMODE]) speakLevel(false);
    if (keybounce[1].rose()) {
        if (levlmode[SPKL_SPKMODE]) speakLevel(true);
        return;
    }
    if (lastMainMode != mainMode || lastSubMode != subMode) {
        lastMainMode = mainMode;
        lastSubMode = subMode;
        announceLevel();
    }
    if (keybounce[0].rose()) {
        subMode = (subMode + 1) % SPKL_MAX;
        return;
    }
    if (keybounce[2].rose()) {
        levlmode[subMode] = !levlmode[subMode];
        announceLevel();
        return;
    }
    
    
}


void doSpeakLidar(void)
{
    int z,d;
    if (!lidrmode[SPKA_SPKMODE]) speakLidar(false);
    if (lastMainMode != mainMode || lastSubMode != subMode) {
        lastMainMode = mainMode;
        lastSubMode = subMode;
        announceLidar();
    }
    if (keybounce[1].rose()) {
        if (lidrmode[SPKA_SPKMODE]) speakLidar(true);
        /*
        else if (subMode == SPKA_CORRECT) {
            zeroGyro();
            saycst("Żyroskop wyzerowany");
        }
        */
        return;
    }
    if (keybounce[0].rose()) {
        subMode = (subMode + 1) % SPKA_MAX;
        return;
    }
    if (keybounce[2].rose()) {
        switch (subMode) {
            case SPKA_CORRECT:
            lidrmode[SPKA_CORRECT] = (lidrmode[SPKA_CORRECT]+1) %3;
            announceLidar();
            break;
            
            case SPKA_DISTAN:
            d = (lidrmode[SPKA_DISTAN] + 1) % 3;
            z = setLidarDistan(d);
            if (z) {
                saycst("Błąd ustawienia dystansu");
                
            }
            else {
                lidrmode[SPKA_DISTAN] = d;
                announceLidar();
            }
            break;

#ifdef HAVE_NARROW
            case SPKA_SPKANGLE:
            z=setNarrowAngle(lidrmode[SPKA_SPKANGLE]);
            if (z) {
                saycst("Błąd odczytu czujnika");
                break;
            }

            // pass trhu
#endif
            default:
            lidrmode[subMode]=!lidrmode[subMode];
            announceLidar();
        }
        return;
    }
    
        
        
    
}

void sayBattery(void)
{
    sayfmt("Bateria %d %%", getBatteryStatus(NULL));
}

uint32_t countdown_ti;
int countdown;
void startCountdown(void)
{
    countdown = 0;
    countdown_ti = millis();
}

int doCountdown(void)
{
    uint32_t delta = millis() - countdown_ti;
    if (delta > 5000) {
        return 1;
    }
    if (delta  > 1000 * countdown) {
        sayfmt("%d", 5 - countdown);
        countdown++;
    }
    return 0;
}

        

void doSettingsLev(void)
{
    static int x, y, z, x2, y2, z2;
    switch(settingSub) {
        case 0:
        saycst("Kalibracja poziomicy. Postaw urządzenie na płaskiej i w miarę poziomej powierzchni i naciśnij lewy klawisz kiedy będziesz gotowy, lub prawy aby przejść do kalibracji pionu.");
        settingSub = SELV_WAIT1;

        case SELV_WAIT1:
        if (keybounce[0].rose()) {
            settingSub = SELV_EOC2;
            return;
        }
            
        if (!keybounce[2].rose()) return;
        settingSub = SELV_COUNTDOWN1;
        startCountdown();

        case SELV_COUNTDOWN1:
        if (doCountdown()) {
            startLevCalibration();
            settingSub = SELV_CALIBXY;
        }
        break;

        case SELV_CALIBXY:
            if (!readLevCalibrationData(&x, &y, &z)) break;
            saycst("Obróć urządzenie o 180 stopni i wciśnij lewy klawisz");
            settingSub = SELV_WAIT2;

        case SELV_WAIT2:
        if (!keybounce[2].rose()) return;
        settingSub = SELV_COUNTDOWN2;
        startCountdown();
        case SELV_COUNTDOWN2:
        if (doCountdown()) {
            startLevCalibration();
            settingSub = SELV_CALIBXY2;
        }
        break;

        case SELV_CALIBXY2:
            if (!readLevCalibrationData(&x2, &y2, &z2)) break;
            saycst("Kalibracja pozioma zakończona. Naciśnij środkowy klawisz aby zapisać dane");
            settingSub = SELV_EOC;

        case SELV_EOC:
            if (keybounce[1].rose()) {
                storeLevCalibDataXY((x+x2)/2, (y+y2) / 2);
                saycst("Dane zostały zapisane");
                settingSub = SELV_EOC1;
            }
            break;

        case SELV_EOC1:
            if (!isSpeaking()) settingSub = SELV_EOC2;
            break;

        case SELV_EOC2:
            saycst("Kalibracja osi pionowej. Przyłóż urządzenie pionowo do ściany i naciśnuj lewy klawisz kiedy będziesz gotowy, lub prawy jeśli chcesz zrezygnować z kalibracji");
            settingSub = SELV_WAIT3;

        case SELV_WAIT3:
        if (keybounce[0].rose()) {
            settingSub = 0;
            return;
        }
            
        if (!keybounce[2].rose()) return;
        settingSub = SELV_COUNTDOWN3;
        startCountdown();

        case SELV_COUNTDOWN3:
        if (doCountdown()) {
            startLevCalibration();
            settingSub = SELV_CALIBZ;
        }
        break;

        case SELV_CALIBZ:
            if (!readLevCalibrationData(&x, &y, &z)) break;
            saycst("Obróć urządzenie o 180 stopni i wciśnij lewy klawisz");
            settingSub = SELV_WAIT4;
            
        case SELV_WAIT4:
        if (keybounce[0].rose()) {
            settingSub = 0;
            return;
        }
            
        if (!keybounce[2].rose()) return;
        settingSub = SELV_COUNTDOWN4;
        startCountdown();
        
        case SELV_COUNTDOWN4:
        if (doCountdown()) {
            startLevCalibration();
            settingSub = SELV_CALIBZ2;
        }
        break;

        case SELV_CALIBZ2:
            if (!readLevCalibrationData(&x2, &y2, &z2)) break;
            saycst("Kalibracja zakończona. Naciśnij środkowy klawisz aby zapisać dane");
            settingSub = SELV_EOZ;

        case SELV_EOZ:

            if (keybounce[1].rose()) {
                storeLevCalibDataZ((z+z2)/2);
                saycst("Dane zostały zapisane");
                settingSub = SELV_FINALE;
            }
            break;
        case SELV_FINALE:
            if (!isSpeaking()) settingMode = false;
            
    }
}

void doSettingsGyro(void)
{
    switch(settingSub) {
        case SEGY_START:
        saycst("Kalibracja żyroskopu. Ustaw urządzenie na płaskiej poziomej powierzchni i wciśnij lewy klawisz aby rozpocząć kalibrację");
        settingSub=SEGY_WAIT;

        case SEGY_WAIT:
        if (keybounce[2].rose()) {
            startCountdown();
            settingSub = SEGY_COUNTDOWN;
        }
        break;

        case SEGY_COUNTDOWN:
        if (doCountdown()) {
            startGyroCalibration();
            settingSub = SEGY_CALIBRATE;
        }
        break;

        case SEGY_CALIBRATE:
        if (calibrateGyro()) {
            saycst("Dane kalibracyjbne zostały zapisane");
            settingSub = SEGY_FINALE;
        }
        break;

        case SEGY_FINALE:
        if (!isSpeaking()) settingMode = false;
    }
}
        
void doSettingsLid(void)
{
    switch (settingSub) {
        case 0:
        saycst("Kalibracja czujnika laserowego. Jeśli chcesz usunąć dane kalibracji, naciśnij prawy klawisz. Jeśli jesteś gotowy, naciśnij lewy klawisz. Uwaga: musisz zapoznać się z instrukcją przed wykonaniem tej czynności.");
        settingSub = SELA_WAIT1;

        case SELA_WAIT1:
        if (keybounce[2].rose()) settingSub = SELA_SPKCAL1;
        else if (keybounce[0].rose()) {
            saycst("Naciśnij lewy klawisz aby potwierdzić usunięcie danych");
            settingSub = SELA_CONFIRM;
        }
        return;

        case SELA_CONFIRM:
        if (keybounce[2].rose()) {
            removeLidarCalib();
            saycst("Dane zostały usunięte");
            settingSub = SELA_FINALE;
        }
        return;

        case SELA_SPKCAL1:
        saycst("Kalibracja matrycy. Naciśnij lewy klawisz kiedy będziesz gotowy, lub prawy aby przejść do kalibracji offsetu.");
        settingSub = SELA_WAITCAL1;

        case SELA_WAITCAL1:
        if (keybounce[2].rose()) settingSub = SELA_WAITCAL1A;
        else if (keybounce[0].rose()) settingSub = SELA_SPADOK1;
        break;

        case SELA_WAITCAL1A:
        saycst("Upewnij się, że w odległości 5 cm od czujnika nie znajduje się żaden przedmiot i naciśnij lewy klawisz");
        settingSub = SELA_WAITCAL1B;

        case SELA_WAITCAL1B:
        if (keybounce[2].rose()) {
            startCountdown();
            settingSub = SELA_COUNTDOWN1;
        }
        break;

        case SELA_COUNTDOWN1:
            if (!doCountdown()) return;
        
            if (calibrateSPAD()) {
                saycst("Błąd kalibracji, wciśnij lewy klawisz aby powtórzyć");
                settingSub = SELA_WAITCAL1B;
            }
            else {
                saycst("Kalibracja matrycy wykonana. Naciśnij środkowy przycisk aby zapisać dane");
                settingSub = SELA_SPADOK;
            }
        
        
        break;

        case SELA_SPADOK:
        if (keybounce[1].rose()) {
            storeCalibration();
            saycst("Kalibracja matrycy zapisana");
            settingSub = SELA_SPADOK1;
        }
        break;

        case SELA_SPADOK1:
        if (isSpeaking()) return;
        saycst("Offset. Umieść kartę kalibracyjną w odległości 140mm od czoła urządzenia i naciśnij lewy klawisz, lub naciśnik prawy klawisz aby przejść do kalibracji przesłuchu");
        settingSub = SELA_SPADOK2;

        case SELA_SPADOK2:
        if (keybounce[0].rose()) settingSub=SELA_OFFOK1;
        else if (keybounce[2].rose()) {
            startCountdown();
            settingSub=SELA_CALIBOF;
        }
        break;

        case SELA_CALIBOF:
        if (!doCountdown()) return;
            if (calibrateSPAD()) {
                saycst("Błąd kalibracji, wciśnij lewy klawisz aby powtórzyć");
                settingSub = SELA_SPADOK2;
            }
            else {
                saycst("Kalibracja offsetu wykonana. Naciśnij środkowy przycisk aby zapisać dane");
                settingSub = SELA_OFFOK;
            }
        break;

        case SELA_OFFOK:
        if (keybounce[1].rose()) {
            storeCalibration();
            saycst("Kalibracja offsetu zapisana");
            settingSub = SELA_OFFOK1;
        }
        break;

        case SELA_OFFOK1:
        if (isSpeaking()) return;
        sayfmt("Przesłuch. Umieść kartę kalibracyjną w odległości %dmm od czoła urządzenia i naciśnij lewy klawisz", crosstalk);
        settingSub = SELA_OFFOK2;

        case SELA_OFFOK2:
        if (!keybounce[2].rose()) return;
        startCountdown();

        settingSub = SELA_CROCNT;

        case SELA_CROCNT:
        if (!doCountdown()) return;
        if (calibrateCross()) {
                saycst("Błąd kalibracji, wciśnij lewy klawisz aby powtórzyć");
                settingSub = SELA_OFFOK2;
            }
            else {
                saycst("Kalibracja przesłuchu wykonana. Naciśnij środkowy przycisk aby zapisać dane");
                settingSub = SELA_DONE;
            }
        break;

        case SELA_DONE:
        if (keybounce[1].rose()) {
            storeCalibration();
            saycst("Kalibracja przesłuchu zapisana");
            settingSub = SELA_FINALE;
        }
        break;

        case SELA_FINALE:
        if (!isSpeaking()) settingMode = false;

    }
}

struct prefCurState {
    unsigned int m_mode:2;
    unsigned int lev_mode:1;
    unsigned int lev_axis:1;
    unsigned int lid_mode: 1;
    unsigned int lid_simple:1;
    unsigned int lid_narrow:1;
    unsigned int lid_point:1;
    unsigned int lid_distan:2;
    unsigned int lid_correct:2;
};


        
static void storeCurState(void)
{
    struct prefCurState prefCurState;
    prefCurState.m_mode = mainMode;
    prefCurState.lev_mode = levlmode[0];
    prefCurState.lev_axis = levlmode[1];
    prefCurState.lid_mode = lidrmode[SPKA_SPKMODE];
    prefCurState.lid_simple = lidrmode[SPKA_SPKSIMPLE];
#ifdef HAVE_NARROW
    prefCurState.lid_narrow = lidrmode[SPKA_SPKANGLE];
#endif
    prefCurState.lid_point = lidrmode[SPKA_POINT];
    prefCurState.lid_distan = lidrmode[SPKA_DISTAN];
    prefCurState.lid_correct = lidrmode[SPKA_CORRECT];
    preferences.putBytes("curstate",&prefCurState,sizeof(prefCurState));
    saycst("Stan został zapisany");
    
}

static void restoreCurState(void)
{
    struct prefCurState prefCurState;
    if (preferences.getBytes("curstate",&prefCurState,sizeof(prefCurState)) != sizeof(prefCurState)) {
        return;
    }
    mainMode = prefCurState.m_mode;
    levlmode[0] = prefCurState.lev_mode;
    levlmode[1] = prefCurState.lev_axis;
    lidrmode[SPKA_SPKMODE] = prefCurState.lid_mode;
    lidrmode[SPKA_SPKSIMPLE] = prefCurState.lid_simple;
#ifdef HAVE_NARROW
    lidrmode[SPKA_SPKANGLE] = prefCurState.lid_narrow;
#endif
    lidrmode[SPKA_POINT] = prefCurState.lid_point;
    lidrmode[SPKA_DISTAN] = prefCurState.lid_distan;
    lidrmode[SPKA_CORRECT] = prefCurState.lid_correct;
}


uint8_t lastSettingMode=0;
void loop(void)
{
    int j;bool cng;
    static bool changemode=false;
    static bool zerog=false;
    for (j=0, cng=false; j<3; j++) {
        cng |= keybounce[j].update();
    }
    if (doSerialConversation()) return;
    if (cng) {
        lastUsage=millis();
    }
    else if (anoBatt < MAX_ANOBAT - 1) {
        uint32_t delta = 0;
#ifdef PIN_VOLT
        if (anoBatt == 2) { // naładowana
            int n = getBatteryStatus(NULL);
            
            if (n < 40) delta = 120000UL;
            else if (n < 60) delta = 600000UL;
        }
        else {
#endif
            if (!anoBatt) delta = 120000UL;
            else delta = 600000UL;
#ifdef PIN_VOLT
        }
#endif    
        if (delta && millis() - lastUsage >= 600000UL) {
#ifdef PIN_VOLT
            sayBattery();
#else        
            say("Urządzenie włączone");
#endif
            lastUsage=millis();
        }
    }
    readBattery();
    
    lidarLoop();
    if (!settingMode)
        readLevel();
    //do_SpeechMenu();

    
    // zmiana ustawienia/praca niezależnie od trybu
    //
    if (changemode) {
        if (keybounce[0].read() && keybounce[2].read()) changemode = false;
        return;
    }
    if (!keybounce[0].read() && !keybounce[2].read()) {
        changemode = true;
        settingMode = !settingMode;
        if (settingMode) {
            settingMenu = 0;
            settingSub = 0;
            settingMenuLast = 0xff;
            settingSubLast = 0xff;
        }
        else {
            mainMode = 0;
            subMode = 0;
            lastMainMode = 0xff;
            lastSubMode = 0xff;
        }
        return;
    }
    if (settingMode != lastSettingMode) {
        lastSettingMode = settingMode;
        stopAudio();
        while (isSpeaking()) delay(1);
        stopLidar(settingMode != 0);
        if (!settingMode) {
            setLidarDistan(lidrmode[SPKA_DISTAN]);
#ifdef HAVE_NARROW
            setNarrowAngle(!lidrmode[SPKA_SPKANGLE]);
#endif
        }
        
    }
    if (settingMode != lastSettingMode || subMode != lastSubMode) {
        zerog = false;
    }

    if (keybounce[1].rose() && keybounce[1].previousDuration() > 750) {
        if (settingMode) {
            settingMenu = (settingMenu + 1) % SMEN_MAX;
            settingSub = 0;
        }
        else {
            mainMode = (mainMode + 1) % 2;
            subMode = 0;
        }
        return;
    }
    if (!settingMode && mainMode &&  lidrmode[SPKA_CORRECT] == 2) {
        if (!keybounce[2].read() && !keybounce[1].read()) {
            zerog = true;
            return;
        }
        if (zerog) {
            if (keybounce[2].read() && keybounce[1].read()) {
                zerog = false;
                zeroGyro();
                saycst("Żyroskop wyzerowany");
            }
            return;
        }
    }
    if (!settingMode && (keybounce[0].fell() || keybounce[1].fell() || keybounce[2].fell())) {
        stopAudio();
        return;
    }
#ifdef PIN_VOLT    
    // stan baterii
    if (!settingMode && keybounce[0].rose() && keybounce[0].previousDuration() > 750) {
        sayBattery();
        return;
    }
#endif
    if (!settingMode && keybounce[2].rose() && keybounce[2].previousDuration() > 750) {
        storeCurState();
        return;
    }
    //doSpeakLidar();
    if (settingMode) {
        switch (settingMenu) {
            case SMEN_SPEECH:
                do_SpeechMenu();
                break;
            case SMEN_PRON:
                doSettingsPron();
                break;
            case SMEN_LEVEL:
                doSettingsLev();
                break;
            case SMEN_GYRO:
                doSettingsGyro();
                break;
#ifdef USE_GLASS
            case SMEN_LASER:
                doSettingsLid();
                break;
#endif
        }
    }
    else {
        if (mainMode) doSpeakLidar();
        else doSpeakLevel();
    }
}
