#include "Poziomica.h"

/*
 * Based on idea from 18650CL library by Pangodream
 * https://github.com/pangodream/18650CL
 */

static uint16_t real_volt = REAL_VOLT;
static uint16_t adc_volt = ADC_VOLT;

#ifdef PIN_VOLT
static const int16_t voltlevel[]={
    3250, 3300, 3350, 3400, 3450,
    3500, 3550, 3600, 3650, 3700,
    3703, 3706, 3710, 3713, 3716,
    3719, 3723, 3726, 3729, 3732,
    3735, 3739, 3742, 3745, 3748,
    3752, 3755, 3758, 3761, 3765,
    3768, 3771, 3774, 3777, 3781,
    3784, 3787, 3790, 3794, 3797,
    3800, 3805, 3811, 3816, 3821,
    3826, 3832, 3837, 3842, 3847,
    3853, 3858, 3863, 3868, 3874,
    3879, 3884, 3889, 3895, 3900,
    3906, 3911, 3917, 3922, 3928,
    3933, 3939, 3944, 3950, 3956,
    3961, 3967, 3972, 3978, 3983,
    3989, 3994, 4000, 4008, 4015,
    4023, 4031, 4038, 4046, 4054,
    4062, 4069, 4077, 4085, 4092,
    4100, 4111, 4122, 4133, 4144,
    4156, 4167, 4178, 4189, 4200};

static int ladc = 0,batLevel=0,lvolt=0;
#endif

static int setvolt;

void readBattery(void)
{
#ifdef PIN_VOLT
    static int adc = 0;
    static int adcstep = 0;
    if (adcstep < 64) {
        adc += analogRead(PIN_VOLT);
        adcstep++;
        return;
    }
#ifdef PRINT_ADC
    {
        static uint32_t lastprint=millis();
        if (millis() - lastprint > 1000UL) {
            printf("ADC volt %d\n", analogRead(PIN_VOLT));
            lastprint = millis();
        }
    }
#endif
    if (setvolt) {
        uint16_t data[2];
        data[0] = real_volt = setvolt;
        data[1] = adc_volt = adc / 64;
        preferences.putBytes("realvolt",data,4);
        setvolt = 0;
    }
        
        
    if (!ladc) ladc = adc /64;
    else ladc = (ladc * 9) / 10 + adc / 640;
    lvolt = (ladc * REAL_VOLT) / ADC_VOLT;
    int i;
    for (i=100; i>0;i--) {
        if (lvolt >= voltlevel[i-1])  break;
    }
    batLevel=i;
#endif
}

int8_t getBatteryStatus(int16_t *volt)
{
#ifdef PIN_VOLT
    if (volt) * volt = lvolt;
    return batLevel;
#else
    if (volt) *volt=0;
    return -1;
#endif
}

void initBattery(void)
{
    uint16_t data[2];
    if (preferences.getBytes("realvolt",data,4) == 4) {
        real_volt = data[0];
        adc_volt = data[1];
    }
}

void setBattery(int milivolt)
{
    setvolt = milivolt;
}
void resetVolt(void)
{
#ifdef PIN_VOLT
    preferences.remove("realvolt");
    real_volt = REAL_VOLT;
    adc_volt = ADC_VOLT;
#endif
}

int getVoltClb(int *rlv, int *adv, int *cra)
{

#ifdef PIN_VOLT
    *cra = analogRead(PIN_VOLT);
    *rlv = real_volt;
    *adv = adc_volt;
    return (*cra * real_volt) / (adc_volt);
#else
    return 0;
#endif
}

