#include <Wire.h>
#include "Poziomica.h"
#include <math.h>

const int MPU = 0x68;

void scanner()
{
  byte error, address;
  int nDevices;

  Serial.println("Scanning...");

  nDevices = 0;
  for(address = 1; address < 127; address++ ) 
  {
    // The i2c_scanner uses the return value of
    // the Write.endTransmisstion to see if
    // a device did acknowledge to the address.
    Wire.beginTransmission(address);
    error = Wire.endTransmission();

    if (error == 0)
    {
      Serial.print("I2C device found at address 0x");
      if (address<16) 
        Serial.print("0");
      Serial.print(address,HEX);
      Serial.println("  !");

      nDevices++;
    }
    else if (error==4) 
    {
      Serial.print("Unknow error at address 0x");
      if (address<16) 
        Serial.print("0");
      Serial.println(address,HEX);
    }
    else Serial.println(address,HEX);  
  }
  if (nDevices == 0)
    Serial.println("No I2C devices found\n");
  else
    Serial.println("done\n");

  delay(5000);           // wait 5 seconds for next scan
}

static void writeReg(uint8_t reg, uint8_t val)
{
    Wire.beginTransmission(MPU);
    Wire.write(reg);
    Wire.write(val);
    Wire.endTransmission(true);
}

static uint8_t readReg(uint8_t reg)
{
    Wire.beginTransmission(MPU);
    Wire.write(reg);
    Wire.endTransmission(false);
    Wire.requestFrom(MPU, 1, 1);
    return Wire.read();
}

static int count=0,pos=0,orient=0,vorient=-1;
static float rat,rbt,rct,angle1,angle2;
static float rgx, rgy, rgz;
static float Xcalg=0, Ycalg=0, Zcalg=0;
static int Xcal=0, Ycal=0, Zcal=0;
static float gyangle[3]={0,0,0};

#define MPU6050_ADDR         0x68
#define MPU6050_SMPLRT_DIV   0x19
#define MPU6050_CONFIG       0x1a
#define MPU6050_GYRO_CONFIG  0x1b
#define MPU6050_ACCEL_CONFIG 0x1c
#define MPU6050_WHO_AM_I     0x75
#define MPU6050_PWR_MGMT_1   0x6b
#define MPU6050_TEMP_H       0x41
#define MPU6050_TEMP_L       0x42
#define MPU6050_RA_GYRO_ZOUT_H      0x47
#define MPU6050_INT_PIN_CFG 0x37
#define MPU6050_INT_ENABLE 0x38
#define MPU6050_INT_STATUS 0x3A

int initMPU(void)
{
#if 0
    writeReg(0x6b,0x28);
    writeReg(0x6c,0xc7);
    writeReg(0x38,1);
    writeReg(0x1C,0);
#else
    writeReg(MPU6050_PWR_MGMT_1, 0x03);
    writeReg(MPU6050_SMPLRT_DIV, 0x00);
    writeReg(MPU6050_CONFIG, 0x02);
    writeReg(MPU6050_GYRO_CONFIG, 0x10); // 1000°/s
    writeReg(MPU6050_ACCEL_CONFIG, 0x00);
    //writeReg(MPU6050_INT_PIN_CFG,0x20);
    //writeReg(MPU6050_INT_ENABLE,0x1);
  
#endif
    rat=rbt=rct=angle1=angle2=0.0;
    count=0; pos=0;
    Xcal=preferences.getInt("xcal",0);
    Ycal=preferences.getInt("ycal",0);
    Zcal=preferences.getInt("zcal",0);
    Xcalg=preferences.getFloat("gycx",0.0);
    Ycalg=preferences.getFloat("gycy",0.0);
    Zcalg=preferences.getFloat("gycz",0.0);
    return 1;
    
}

static float lvresult[20][3];

void zeroGyro(void)
{
    gyangle[0] = 0;
    gyangle[1] = 0;
    gyangle[2] = 0;
}

float getGyroZ(int z)
{
    return gyangle[z];
}


int getLevel(float *anglex, float *angley, float *gmpx)
{
    int i,j;
    float x=0, y=0, z=0;
    for (i=0, j= pos; i < count; i++) {
        j = j-1;
        if (j<0) j=19;
        x += lvresult[j][0];
        y += lvresult[j][1];
        z += lvresult[j][2];
    }
    *anglex=(x * 180)/(3.141592*count);
    *angley=(y * 180)/(3.141592*count);
    if (gmpx) *gmpx = (z * 180)/(3.141592*count);
    return vorient;
        
}

void storeLevCalibDataXY(int x, int y)
{
    Xcal=x;
    Ycal=y;
    preferences.putInt("xcal",x);
    preferences.putInt("ycal",y);
}

void storeLevCalibDataZ(int z)
{
    Zcal = z;
    preferences.putInt("zcal",z);
}



static int calibX, calibY, calibZ, calibCNT;
void startLevCalibration(void)
{
    calibX = calibY = calibZ = calibCNT = 0;
}

static int readAvailable(void)
{
#if 0
    uint8_t rc=readReg(0x3a);
    if (!(rc & 1)) return 0;
    return 1;
#else
    static uint32_t msc = micros();
    uint32_t udelta = micros() - msc;
    if (udelta < 15000UL) return -1;
    msc = micros();
    return (int)udelta;
#endif
}

float gycX, gycY, gycZ;
void startGyroCalibration(void)
{
    gycX=0; gycY=0; gycZ = 0;

}

int calibrateGyro(void)
{
    if (readAvailable() < 0) return 0;
    Wire.beginTransmission(MPU);
    Wire.write(0x43); 
    Wire.endTransmission(false);
    Wire.requestFrom(MPU, 6, 1); 
    int16_t GyX = Wire.read() << 8 | Wire.read(); // X-axis value
    int16_t GyY = Wire.read() << 8 | Wire.read(); // Y-axis value
    int16_t GyZ = Wire.read() << 8 | Wire.read(); // Z-axis va
    gycX += GyX / 32.8;
    gycY += GyY / 32.8;
    gycZ += GyZ / 32.8;
    calibCNT++;
    if (calibCNT < 1000) return 0;
    Xcalg = gycX / calibCNT;
    Ycalg = gycY / calibCNT;
    Zcalg = gycZ / calibCNT;
    preferences.putFloat("gycx",Xcalg);
    preferences.putFloat("gycy",Ycalg);
    preferences.putFloat("gycz",Zcalg);
    return 1;
}

int readLevCalibrationData(int *x, int *y, int *z)
{
    int usec = readAvailable();
    if (usec < 0) return 0;
    if (calibCNT >= 40) {
        if (x) *x=calibX / 40 - 32768L;
        if (y) *y=calibY / 40 - 32768L;
        if (z) *z=calibZ / 40 - 32768L;
        return 1;
    }
        
    Wire.beginTransmission(MPU);
    Wire.write(0x3B); // Start with register 0x3B (ACCEL_XOUT_H)
    Wire.endTransmission(false);
    Wire.requestFrom(MPU, 6, 1); // Read 6 registers total, each axis value is stored in 2 registers
    int16_t AcX = Wire.read() << 8 | Wire.read(); // X-axis value
    int16_t AcY = Wire.read() << 8 | Wire.read(); // Y-axis value
    int16_t AcZ = Wire.read() << 8 | Wire.read(); // Z-axis va
    int cX = AcX+32768L;
    int cY = AcY+32768L;
    int cZ = AcZ+32768L;

    calibX += cX;
    calibY += cY;
    calibZ += cZ;
    calibCNT += 1;
    return 0;
}

void readLevel(void)
{
    int usec = readAvailable();
    if (usec < 0) return;
    //("Usec = %d\n", usec);
    Wire.beginTransmission(MPU);
    Wire.write(0x3B); // Start with register 0x3B (ACCEL_XOUT_H)
    Wire.endTransmission(false);
    Wire.requestFrom(MPU, 14, 1); // Read 6 registers total, each axis value is stored in 2 registers
    int16_t AcX = Wire.read() << 8 | Wire.read(); // X-axis value
    int16_t AcY = Wire.read() << 8 | Wire.read(); // Y-axis value
    int16_t AcZ = Wire.read() << 8 | Wire.read(); // Z-axis va
    int16_t temp = Wire.read() << 8 | Wire.read();
    int16_t GyX = Wire.read() << 8 | Wire.read();
    int16_t GyY = Wire.read() << 8 | Wire.read();
    int16_t GyZ = Wire.read() << 8 | Wire.read();

    //printf("AC %d %d %d\n", AcX, AcY, AcZ);

    AcX -= Xcal;
    AcY -= Ycal;
    AcZ -= Zcal;
    int deltax = abs(AcX) - abs(AcY) -abs(AcZ);
    int deltay = abs(AcY) - abs(AcX) -abs(AcZ);
    int deltaz = abs(AcZ) - abs(AcY) -abs(AcX);
    if (deltax > 5000) {orient = 2;if (AcX < 0) orient |= 4;}
    else if (deltay > 5000) {orient = 1;if (AcY > 0) orient |= 4;}
    else if (deltaz > 5000) {orient = 0;if (AcZ < 0) orient |= 4;}
    if (orient != vorient) {
        count = 0;
        vorient=orient;
        pos=0;
    }
    switch (orient & 3) {
        case 4:
        AcX = -AcX;
        // pass thru
        case 0:
        rat = rct = atan2(AcX, sqrt(AcZ*AcZ + AcY * AcY));
        rbt = atan2(AcY, sqrt(AcZ*AcZ + AcX * AcX));
        break;

        case 5:
        AcX = -AcX;
        // pass thru
        case 1:
        
        rat = rct = atan2(AcX, sqrt(AcZ*AcZ + AcY * AcY));
        rbt = atan2(AcZ, sqrt(AcY*AcY + AcX * AcX));
        break;

        default:
        
        rat = atan2(AcZ, sqrt(AcX*AcX + AcY * AcY));
        rbt = atan2(AcY, sqrt(AcZ*AcZ + AcX * AcX));
        rct = atan2(sqrt(AcZ*AcZ + AcY * AcY), abs(AcX));
        
    }
    rgx = ((GyX / 32.8) - Xcalg) * usec * 0.000001;
    rgy = ((GyY / 32.8) - Ycalg) * usec * 0.000001;
    rgz = ((GyZ / 32.8) - Zcalg) * usec * 0.000001;
    //("RGZ = %d %f (%f %f)\n", GyZ, rgz,(GyZ / 32.8),Zcalg);
    lvresult[pos][0] = rat;
    lvresult[pos][1] = rbt;
    lvresult[pos][2] = rct;
    gyangle[0] += rgx;
    gyangle[1] += rgy;
    gyangle[2] += rgz;
    
    pos=(pos+1) % 20;
    if (count < 20) count++;
}

