#include "Poziomica.h"

static char buf[1024];
static int bufpos=0;
static char *tstr;

static int getser(void)
{

    while (Serial.available()) {
        int z=Serial.read();
        if (z == 13 || z == 10) {
            if (!bufpos) continue;
            buf[bufpos]= 0;
            bufpos=0;
            return 1;
        }
        if (bufpos < 1023) buf[bufpos++]=z;
    }
    return 0;
}

static char *getToken(bool required=true)
{
    char *cmd;
    while (*tstr && isspace(*tstr)) tstr++;
    if (!*tstr) {
        if (required) printf("Brak wymaganego parametru\n");
        return NULL;
    }
    cmd = tstr;
    while (*tstr && !isspace(*tstr)) tstr++;
    if (*tstr) {
        *tstr++=0;
        while (*tstr && isspace(*tstr)) tstr++;
    }
    return cmd;
}

static int getUInt(bool required=true, int minval=0, int maxval=9999, int reset = -1)
{
    char *t=getToken(required);
    if (!t) return -1;
    char *t1;
    
    if (!strcmp(t, "reset") && reset >= 0) {
        return reset;
    }
    int ival = strtol(t,&t1,10);
    if (*t1) {
        printf("Błędny zapis liczby: %s\n", t);
        return -2;
    }
    if (ival < minval || ival > maxval) {
        printf("Liczba %d poza zakresem %d .. %d\n", ival, minval, maxval);
        return -2;
    }
    return ival;
}


static void servolt(void)
{
    char *vlt;
    int vt; bool dot=false;
    vlt = getToken(false);
    if (!vlt) {
        int rlv, adv, cra;
        vt = getVoltClb(&rlv, &adv, &cra);
        printf("Napięcie %d mV, odczyt %d, kalibracja %d/%d\n", vt, cra, rlv, adv);
        return;
    }
    if (!strcmp(vlt, "reset")) {
        resetVolt();
        printf("Przywrócono ustawienia pierwotne\n");
        return;
    }
    vt=0;
    for (;*vlt;vlt++) {
        if (*vlt == '.') {
            if (!dot) {
                dot=true;
                continue;
            }
        }
        if (!isdigit(*vlt)) break;
        vt = (10 * vt) + (*vlt - '0');
    }
    if (*vlt) {
        printf("Nieprawidłowy parametr\n");
        return;
    }
    while (vt < 1000) vt *= 10;
    if (vt <3000 || vt > 4500) {
        printf("Napięcie spoza zakresu od 3 do 4.5V\n");
        return;
    }
    setBattery(vt);
    printf("Napięcie kalibracyjne %d mV zostało zapisane\n", vt);
}

static void sayme(void)
{
    if (!*tstr) {
        printf("Co ja miałem powiedzieć?\n");
    }
    else say(tstr);
}

extern int coverdim[5];
extern int std_dims[5];

static void setcover(void)
{
    char *param = getToken(false);
    static const char *parname[] = {
        "długość","szerokość","wysokość","głębokość lasera","wysokość lasera"};
    static uint16_t minmax[5][2]={
        {50,500},
        {50,500},
        {15,500},
        {0,10},
        {5,500}};
    int i;
    if (!param) {
        printf("Wymiary obudowy:\n");
        for (i=0; i<5; i++) printf("%s: %d mm\n", parname[i], coverdim[i]);
        return;
    }
    int num = -1;
    if (!param[1]) {
        for (i=0; i<5; i++) if (*param == "lwhdo"[i]) break;
        if (i < 5) num = i;
    }
    if (num < 0) {
        printf("Nieznany parametr: %s\n", param);
        return;
    }
    int pval=getUInt(true, minmax[num][0], minmax[num][1], std_dims[num]);
    if (pval < 0) return;
    coverdim[num] = pval;
    preferences.putBytes("coverdim", coverdim, sizeof(coverdim));
    printf("Ustawiono %s: %d mm\n", parname[num], pval);
}

static void setxtalk(void)
{
    int ival = getUInt(false, 30, 4000, -1);
    if (ival == -2) return;
    if (ival == -1) {
        printf("Odległość kalibracyjna %d mm\n", crosstalk);
    }
    else {
        crosstalk = ival;
        printf("Ustawiono odległość kalibracyjną %d mm\n", crosstalk);
        preferences.putInt("crosstalk",crosstalk);
    }
}

static void rebootMe(void)
{
    ESP.restart();
}

static void helpme(void);
static struct serialCmd {
    const char *name;
    void (*fun)(void);
    const char *descr;
    const char *params;
    const char *szort;
    
} commands[]={
    {"help", helpme, "Wyświetla pomoc dla podanego polecenia lub wszystkie rozpoznawane polecenia"},
    {"say", sayme, "Przekazuje parametr do syntezatora mowy","<dowolny napis>"},
    {"cover", setcover, "Ustawia wymiary obudowy w milimetrach.\n  Parametry l, w i h to odpowiednio długość, szerokość i wysokość obudowy\n\
  Parametr d to odległość lasera od czoła obudowy (głębokość)\n\
  Parametr o to odległość lasera od dolnej krawędzi obudowy (wysokość)\n\
  Podanie 'reset' zamiast liczby spowoduje przywrócenie domyślnego rozmiaru\n\
  Bez parametrów wyświetla aktualny rozmiar","[l|w|h|d|o <liczba>|reset]","Ustawia wumiary obudowy"},
    {"xtalk",setxtalk,"Wyświetla lub ustawia odległość dla kalibracji crosstalk","odległość"},
#ifdef PIN_VOLT
    {"volt", servolt, "Ustawia bieżące napięcie akumulatora jako odnościk do pomiarów.\n\
  Napięcie może być podawane w dowolny sposób, np. 3.75, 375, 3750 będą uznane za 3.75V\n\
  Podanie \"reset\" przywraca wartości domyślne\n\
  Bez parametru podaje aktualne wartości ustawień\n", "[napięcie w mV]","Kalibracja odczytu baterii"},
#endif
    {"reboot",rebootMe, "Restart mikrokontrolera"},

    {NULL, NULL, NULL}};
    
static void helpme(void)
{
    int i;
    char *cmd=getToken(false);
    if (cmd) {
        for (i=0; commands[i].name; i++) if (!strcmp(cmd, commands[i].name)) {
            if (commands[i].params) {
                printf("%s %s: %s\n",commands[i].name, commands[i].params, commands[i].descr);
            }
            else {
                printf("%s: %s\n", commands[i].name, commands[i].descr);
            }
            return;
        }
        printf("Nierozpoznane polecenie %s\n", cmd);
        return;
    }
                
    printf("Polecenia:\n");
    for (i=0; commands[i].name; i++) {
        printf("%s: %s\n", commands[i].name, commands[i].szort ? commands[i].szort : commands[i].descr);
    }
    printf("---\n");
    return;
}

bool doSerialConversation(void)
{
    if (!getser()) return false;
    tstr = buf;
    char *cmd = getToken(false);
    if (!*cmd) return false;
    int i;
    for (i=0; commands[i].name; i++) {
        if (!strcmp(commands[i].name, cmd)) break;
    }
    if (commands[i].fun) commands[i].fun();
    else printf("Nierozpoznane polecenie '%s', wpisz \"help\" abty poznać polecenia\n", cmd);
    return true;
}
